"""配置模块"""

from .figures import CHARACTER_FIGURE_ACCESSORY_KEYS, CHARACTER_FIGURES
from .profiles import STORY_SCENARIO_STYLES, CHARACTER_POSITIONS

__all__ = [
    'CHARACTER_FIGURES',
    'CHARACTER_FIGURE_ACCESSORY_KEYS',
    'STORY_SCENARIO_STYLES',
    'CHARACTER_POSITIONS'
] 