from typing import Final, List

# 角色形象配件关键词
CHARACTER_FIGURE_ACCESSORY_KEYS: Final[List[str]] = ['mask', 'cap', 'crown', 'olive']

# 角色形象配置
CHARACTER_FIGURES = {
    "cue": [
        "robot-girl-3123483124",
    ],
    "eily": [
        "robot-girl-3123483124",
    ],
    "boy": [
        "boy-stand-arms-fistup-leg-step-blinkgrin-anime",
        "boy-stand-arms-open-leg-open-question-anime",
        "boy-stand-arms-straight-leg-straddle-calm-anime",
        "boy-stand-arms-straight-leg-straight-blinkgrin-anime",
        "boy-stand-arms-straight-leg-straight-blinksay-anime",
        "boy-stand-arms-straight-leg-straight-complain-anime",
        "boy-stand-arms-straight-leg-straight-groan-anime",
        "boy-stand-arms-straight-leg-straight-say-anime",
        "boy-stand-arms-straight-leg-straight-smilesay-anime",
        "boy-stand-left-finger1up-right-straight-leg-crossing-blinkgrin-anime",
        "boy-stand-left-finger1up-right-straight-leg-step-blinkgrin-anime",
        "boy-stand-left-finger1up-right-straight-leg-step-groan-anime",
        "boy-stand-left-finger1up-right-straight-leg-straddle-calm-anime",
        "boy-stand-left-fistup-right-straight-leg-straddle-blinkgrin-anime",
        "boy-stand-left-fistup-right-straight-leg-straddle-complain-anime",
        "boy-stand-left-fistup-right-straight-leg-straddle-groan-anime",
        "boy-stand-left-fistup-right-straight-leg-straddle-smilesay-anime",
        "boy-stand-left-straight-right-handonhip-leg-straddle-calm-anime",
        "boy-stand-left-thumbup-right-handonhip-leg-straddle-smilesay-anime",
        "boy-stand-left-thumbup-right-handonhip-leg-straddle-winking-anime",
        "boy-stand-left-thumbup-right-handonhip-leg-straight-calm-anime",
        "boy-standright-left-handonhip-right-thumbup-leg-straddle-winking-anime",
        "boy-standright-left-straight-right-finger1up-leg-straddle-winking-anime",
        "boy-standright-left-straight-right-fistup-leg-straddle-smilesay-anime",
        "boy-standright-left-straight-right-fistup-leg-straddle-winking-anime",
        "boy-half-left-handonhip-right-finger1up-calm-anime",
        "boy-half-left-handonhip-right-open-smilesay-anime",
        "boy-half-left-handonhip-right-thumbup-winking-anime",
        "boy-head-smilesay-anime",
        "boy-standright-left-straight-right-finger1up-leg-crossing-blinkgrin-anime",
        "boy-stand-arms-straight-leg-straddle-smilesay-anime",
        "boy-stand-arms-straight-leg-straddle-groan-anime",
        "boy-stand-arms-straight-leg-straddle-blinkgrin-anime",
        "boy-stand-left-fistup-right-straight-leg-step-groan-anime",
        "boy-stand-left-fistup-right-straight-leg-step-blinkgrin-anime",
        "boy-stand-left-fistup-right-straight-leg-crossing-blinkgrin-anime",
        "boy-half-left-handonhip-right-thumbup-smilesay-anime",
        "boy-half-arms-straight-winking-anime",
        "boy-stand-left-straight-right-spear-leg-straddle-khoikhoimask-anime",
        "boy-stand-left-fistup-right-spear-leg-straddle-khoikhoimask-anime",
        "boy-stand-left-finger1up-right-straight-leg-crossing-calm-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-straight-right-open-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-thumbup-right-straight-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-questioning-anime",
        "boy-stand-left-finger1up-right-straight-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-straight-worried-anime",
        "boy-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "boy-stand-left-thumbup-right-handonhip-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-laugh-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-apart-blissful-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-thumbup-right-handonhip-torso-shoulder-flat-leg-apart-laugh-anime",
        "boy-stand-left-thumbup-right-open-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-straight-shy-anime",
        "boy-stand-left-finger1up-right-open-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-straight-questioning-anime",
        "boy-stand-left-finger1up-right-straight-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-finger1up-right-open-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-finger1up-right-handonhip-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-straight-right-straight-torso-shoulder-flat-leg-straight-worried-anime",
        "boy-stand-left-fistup-right-handonhip-torso-shoulder-tilt-leg-apart-playful-anime",
        "boy-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-straight-surprised-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "boy-stand-left-finger1up-right-straight-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-thumbup-right-straight-torso-shoulder-flat-leg-apart-laugh-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-apart-laugh-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-thumbup-right-handonhip-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-straight-shy-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-straight-worried-anime",
        "boy-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-straight-worried-anime",
        "boy-stand-left-fistup-right-open-torso-shoulder-tilt-leg-apart-smug-anime",
        "boy-stand-left-fistup-right-handonhip-torso-shoulder-tilt-leg-apart-smug-anime",
        "boy-stand-left-thumbup-right-open-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-straight-surprised-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "boy-stand-left-fistup-right-fistup-torso-shoulder-up-leg-apart-blinkgrin-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-straight-questioning-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-finger1up-right-open-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-fistup-right-straight-torso-shoulder-tilt-leg-apart-playful-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-finger1up-right-handonhip-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-apart-blissful-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-straight-questioning-anime",
        "boy-stand-left-straight-right-straight-torso-shoulder-flat-leg-straight-surprised-anime",
        "boy-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-straight-surprised-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-finger1up-right-handonhip-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-fistup-right-open-torso-shoulder-tilt-leg-apart-blinkgrin-anime",
        "boy-stand-left-thumbup-right-open-torso-shoulder-flat-leg-apart-laugh-anime",
        "boy-stand-left-fistup-right-straight-torso-shoulder-tilt-leg-apart-smug-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-shy-anime",
        "boy-stand-left-fistup-right-handonhip-torso-shoulder-tilt-leg-apart-blinkgrin-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-surprised-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-fistup-right-open-torso-shoulder-tilt-leg-apart-playful-anime",
        "boy-stand-left-thumbup-right-open-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-straight-right-straight-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-blissful-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-straight-shy-anime",
        "boy-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-shy-anime",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-straight-right-straight-torso-shoulder-flat-leg-straight-questioning-anime",
        "boy-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-surprised-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-worried-anime",
        "boy-stand-left-straight-right-straight-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "boy-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-finger1up-right-open-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-thumbup-right-handonhip-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-thumbup-right-straight-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-finger1up-right-handonhip-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-thumbup-right-straight-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-apart-laugh-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-apart-smug-anime",
        "boy-stand-left-fistup-right-straight-torso-shoulder-tilt-leg-apart-blinkgrin-anime",
        "boy-stand-left-thumbup-right-open-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-finger1up-right-open-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-thumbup-right-straight-torso-shoulder-flat-leg-straight-happy-anime",
        "boy-stand-left-finger1up-right-straight-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-straight-right-straight-torso-shoulder-flat-leg-apart-blissful-anime",
        "boy-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-worried-anime",
        "boy-stand-left-straight-right-straight-torso-shoulder-flat-leg-straight-shy-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "boy-stand-left-thumbup-right-handonhip-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-finger1up-right-straight-torso-shoulder-flat-leg-straight-neutral-anime",
        "boy-stand-left-finger1up-right-handonhip-torso-shoulder-flat-leg-apart-playful-anime",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-questioning-anime",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-apart-blissful-anime",
        "boy-stand-left-straight-right-open-torso-shoulder-flat-leg-apart-blissful-anime",
        "boy-stand-left-thumbup-right-handonhip-torso-shoulder-flat-leg-straight-neutral-crown-anime.svg",
        "boy-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-neutral-crown-anime.svg",
        "boy-stand-left-open-right-straight-torso-shoulder-flat-leg-straight-neutral-crown-anime.svg",
        "boy-stand-left-finger1up-right-open-torso-shoulder-flat-leg-straight-neutral-crown-anime.svg",
        "boy-stand-left-finger1up-right-handonhip-torso-shoulder-flat-leg-straight-neutral-crown-anime.svg",
        "boy-stand-left-thumbup-right-straight-torso-shoulder-flat-leg-straight-neutral-crown-anime.svg",
        "boy-stand-left-finger1up-right-straight-torso-shoulder-flat-leg-straight-neutral-crown-anime.svg",
        "boy-stand-left-open-right-open-torso-shoulder-flat-leg-straight-neutral-crown-anime.svg"
    ],
    "girl": [
        "girl-stand-arms-fistup-leg-open-blinkgrin-anime",
        "girl-stand-arms-handonhip-leg-straight-calm-anime",
        "girl-stand-arms-handonhip-leg-straight-complain-anime",
        "girl-stand-arms-handonhip-leg-straight-smilesay-anime",
        "girl-stand-arms-handonhip-leg-straight-winking-anime",
        "girl-stand-arms-open-leg-straight-bemused-anime",
        "girl-stand-arms-open-leg-straight-blinkgrin-anime",
        "girl-stand-arms-open-leg-straight-question-anime",
        "girl-stand-arms-open-leg-straight-smilesay-anime",
        "girl-stand-arms-straight-leg-straight-calm-anime",
        "girl-stand-left-handonhip-right-thumbup-leg-open-winking-anime",
        "girl-stand-left-straight-right-finger1up-leg-crossing-question-anime",
        "girl-stand-left-straight-right-finger1up-leg-straight-groan-anime",
        "girl-stand-left-straight-right-finger1up-leg-straight-smilesay-anime",
        "girl-stand-left-straight-right-finger1up-leg-straight-winking-anime",
        "girl-stand-left-handonhip-right-open-leg-ontiptoe-groan-anime",
        "girl-half-left-handonhip-right-finger1up-smilesay-anime",
        "girl-half-left-handonhip-right-open-smilesay-anime",
        "girl-half-left-handonhip-right-thumbup-winking-anime",
        "girl-head-smilesay-anime",
        "girl-stand-left-shield-right-open-leg-straight-khoikhoimask-anime",
        "girl-stand-left-shield-right-finger1up-leg-straight-khoikhoimask-anime",
        "girl-stand-arms-handonhip-leg-straight-groan-captaincap-anime",
        "girl-stand-left-open-right-finger1up-leg-straight-smilesay-captaincap-anime",
        "girl-stand-left-open-right-finger1up-leg-straight-winking-captaincap-anime",
        "girl-stand-arms-open-leg-straight-captaincap-anime",
        "girl-stand-left-handonhip-right-finger1up-torso-shoulder-flat-leg-apart-smug-anime",
        "girl-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-straight-questioning-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "girl-stand-left-open-right-thumbup-torso-shoulder-flat-leg-apart-laugh-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-laugh-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-handonhip-right-handonhip-torso-shoulder-flat-leg-apart-smug-anime",
        "girl-stand-left-handonhip-right-thumbup-torso-shoulder-flat-leg-apart-laugh-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-surprised-anime",
        "girl-stand-left-handonhip-right-finger1up-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-apart-blissful-anime",
        "girl-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-handonhip-right-handonhip-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-apart-smug-anime",
        "girl-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-open-right-finger1up-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-apart-laugh-anime",
        "girl-stand-left-open-right-thumbup-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-handonhip-right-handonhip-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-open-right-finger1up-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-apart-laugh-anime",
        "girl-stand-left-open-right-thumbup-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-apart-blissful-anime",
        "girl-stand-left-open-right-thumbup-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-apart-laugh-anime",
        "girl-stand-left-open-right-finger1up-torso-shoulder-flat-leg-apart-smug-anime",
        "girl-stand-left-handonhip-right-finger1up-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-handonhip-right-thumbup-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-open-right-thumbup-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-smug-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-apart-smug-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-open-right-finger1up-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-straight-surprised-anime",
        "girl-stand-left-handonhip-right-finger1up-torso-shoulder-flat-leg-straight-questioning-anime",
        "girl-stand-left-fistup-right-fistup-torso-shoulder-up-leg-apart-blinkgrin-anime",
        "girl-stand-left-fistup-right-fistup-torso-shoulder-up-leg-apart-blissful-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-straight-shy-anime",
        "girl-stand-left-handonhip-right-thumbup-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-apart-smug-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-shy-anime",
        "girl-stand-left-handonhip-right-handonhip-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-fistup-right-fistup-torso-shoulder-up-leg-apart-laugh-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "girl-stand-left-handonhip-right-finger1up-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-handonhip-right-handonhip-torso-shoulder-flat-leg-apart-laugh-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-questioning-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-apart-laugh-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-straight-surprised-anime",
        "girl-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-apart-smug-anime",
        "girl-stand-left-open-right-handonhip-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-handonhip-right-thumbup-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "girl-stand-left-handonhip-right-handonhip-torso-shoulder-flat-leg-apart-playful-anime",
        "girl-stand-left-open-right-finger1up-torso-shoulder-flat-leg-straight-questioning-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-straight-questioning-anime",
        "girl-stand-left-handonhip-right-thumbup-torso-shoulder-flat-leg-straight-happy-anime",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-straight-embarrassed-anime",
        "girl-stand-left-open-right-finger1up-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-handonhip-right-finger1up-torso-shoulder-flat-leg-straight-neutral-anime",
        "girl-stand-left-open-right-thumbup-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-handonhip-right-open-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-handonhip-right-thumbup-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-straight-right-handonhip-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-handonhip-right-finger1up-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-open-right-handonhip-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-open-right-open-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-handonhip-right-handonhip-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-open-right-finger1up-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
        "girl-stand-left-straight-right-open-torso-shoulder-flat-leg-straight-happy-olive-anime.svg",
    ],
    "sports-boy": [
        "sports-boy-half-arms-open-cry-anime",
        "sports-boy-half-left-handonhip-right-open-cry-anime",
        "sports-boy-half-left-handonhip-right-fistup-winking-anime",
        "sports-boy-half-left-handonhip-right-handup-winking-anime",
        "sports-boy-half-left-handonhip-right-thumbup-winking-anime",
        "sports-boy-stand-left-fistup-right-straight-leg-straddle-blinkgrin-anime",
        "sports-boy-stand-left-fistup-right-straight-leg-straddle-complain-anime",
        "sports-boy-stand-left-fistup-right-straight-leg-straddle-winking-anime",
        "sports-boy-stand-left-fistup-right-straight-leg-straddle-smilesay-anime",
        "sports-boy-stand-left-fistup-right-straight-leg-straddle-calm-anime",
        "sports-boy-standright-left-straight-right-fistup-leg-straddle-calm-anime",
        "sports-boy-stand-arms-open-leg-straddle-complain-anime",
        "sports-boy-stand-arms-open-leg-straddle-question-anime",
        "sports-boy-stand-arms-open-leg-straddle-cry-anime",
        "sports-boy-stand-left-open-right-straight-leg-straddle-calm-anime",
        "sports-boy-stand-left-handup-right-holdball-leg-walk-winking-anime",
        "sports-boy-stand-left-straight-right-holdball-leg-straddle-winking-anime",
        "sports-boy-stand-left-open-right-holdball-leg-straddle-calm-anime",
        "sports-boy-stand-left-open-right-holdball-leg-straddle-embarrassing-anime",
        "sports-boy-stand-left-thumbup-right-holdball-leg-straddle-embarrassing-anime",
        "sports-boy-stand-left-thumbup-right-holdball-leg-straddle-blinkgrin-anime",
        "sports-boy-head-calm-anime",
        "sports-boy-head-question-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-stepping-surprised-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-apart-blissful-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-stepping-surprised-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-stepping-embarrassed-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-stepping-questioning-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-stepping-surprised-anime",
        "sportsboy-standright-left-fistup-right-fistup-torso-shoulder-up-leg-apart-blinkgrin-anime",
        "sportsboy-standright-left-holding-basketball-right-fistup-torso-shoulder-tilt-leg-apart-smug-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-apart-smug-anime",
        "sportsboy-standright-left-handonhip-right-fistup-torso-shoulder-tilt-leg-apart-smug-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-stepping-happy-anime",
        "sportsboy-standright-left-handonhip-right-fistup-torso-shoulder-tilt-leg-apart-blinkgrin-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-apart-smug-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-stepping-questioning-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-stepping-embarrassed-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-apart-laugh-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-stepping-shy-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-apart-smug-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-apart-blissful-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-stepping-shy-anime",
        "sportsboy-standright-left-holding-basketball-right-fistup-torso-shoulder-tilt-leg-apart-blinkgrin-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-stepping-happy-anime",
        "sportsboy-standright-left-straight-right-fistup-torso-shoulder-tilt-leg-apart-smug-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-apart-laugh-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-apart-blinkgrin-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-stepping-embarrassed-anime",
        "sportsboy-standright-left-straight-right-fistup-torso-shoulder-tilt-leg-apart-blinkgrin-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-stepping-questioning-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-stepping-happy-anime",
        "sportsboy-standright-left-open-right-open-torso-shoulder-flat-leg-apart-laugh-anime",
        "sportsboy-standright-left-straight-right-open-torso-shoulder-flat-leg-stepping-shy-anime",
        "sportsboy-standright-left-straight-right-straight-torso-shoulder-flat-leg-apart-blissful-anime",
    ]
}

# 导出所有常量
__all__ = [
    'CHARACTER_FIGURES',
    'CHARACTER_FIGURE_ACCESSORY_KEYS'
]